"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = combineReducers;
var _lodash = require("lodash");
var _immutable = require("immutable");
var _mindfrontReduxUtils = require("mindfront-redux-utils");
function combineReducers(reducers) {
  var createInitialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _immutable.Map;
  var actionHandlers = {};
  var otherReducers = {};
  var initialStateObj = {};

  // regroup reducers that have actionHandlers:
  //   from: prop name -> action type -> reducer
  //     to: action type -> prop name -> reducer
  // put reducers without actionHandlers into a prop name -> reducer map

  (0, _lodash.forEach)(reducers, function (reducer, key) {
    initialStateObj[key] = reducer(undefined, {});
    if (reducer.actionHandlers) {
      (0, _lodash.forEach)(reducer.actionHandlers, function (actionHandler, type) {
        ;
        (actionHandlers[type] || (actionHandlers[type] = {}))[key] = actionHandler;
      });
    } else otherReducers[key] = reducer;
  });
  var initialState = createInitialState(initialStateObj);

  // now we can create a more efficient reducer for each action type
  // and one more efficient reducer for the otherReducers

  // creates an efficient reducer from a prop name -> reducer map
  var combineBase = function combineBase(reducers) {
    var result;
    if ((0, _lodash.size)(reducers) > 1) {
      // we have to update multiple keys; use state.withMutations
      result = function result() {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments.length > 1 ? arguments[1] : undefined;
        return state.withMutations(function (mutableState) {
          return (0, _lodash.reduce)(reducers, function (nextState, reducer, key) {
            return nextState.update(key, function (value) {
              return reducer(value, action);
            });
          }, mutableState);
        });
      };
    } else {
      // we only have to update one key; use state.update
      var key = Object.keys(reducers)[0];
      var reducer = reducers[key];
      result = function result() {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments.length > 1 ? arguments[1] : undefined;
        return state.update(key, function (value) {
          return reducer(value, action);
        });
      };
    }
    result.domainHandlers = reducers;
    return result;
  };
  var actionHandlerReducer = (0, _lodash.size)(actionHandlers) ? (0, _mindfrontReduxUtils.createReducer)(initialState, (0, _lodash.mapValues)(actionHandlers, combineBase)) : undefined;
  var otherReducer = (0, _lodash.size)(otherReducers) ? combineBase(otherReducers) : undefined;
  if (actionHandlerReducer && otherReducer) return (0, _mindfrontReduxUtils.composeReducers)(actionHandlerReducer, otherReducer);
  if (actionHandlerReducer) return actionHandlerReducer;
  if (otherReducer) return otherReducer;
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    return state;
  };
}
module.exports = exports.default;